@@ -4,7 +4,7 @@ import os
 import abc
 import base64
 
-from keyring.py27compat import configparser
+from six.moves import configparser
 
 from keyring.errors import PasswordDeleteError
 from keyring.backend import KeyringBackend
@@ -77,27 +77,28 @@ class Keyring(FileBacked, KeyringBackend):
     @abc.abstractmethod
     def encrypt(self, password, assoc = None):
         """
-        Given a password (byte string), return an encrypted byte string.
+        Given a password (byte string) and assoc (byte string, optional),
+        return an encrypted byte string.
 
-        assoc might provide associated data (typically: service and username)
+        assoc provides associated data (typically: service and username)
         """
 
     @abc.abstractmethod
     def decrypt(self, password_encrypted, assoc = None):
         """
-        Given a password encrypted by a previous call to `encrypt`, return
-        the original byte string.
+        Given a password encrypted by a previous call to `encrypt`, and assoc
+        (byte string, optional), return the original byte string.
 
-        assoc might provide associated data (typically: service and username)
+        assoc provides associated data (typically: service and username)
         """
 
     def get_password(self, service, username):
         """
         Read the password from the file.
         """
+        assoc = self._generate_assoc(service, username)
         service = escape_for_ini(service)
         username = escape_for_ini(username)
-        assoc = (service + '\0' + username).encode()
 
         # load the passwords from the file
         config = configparser.RawConfigParser()
@@ -122,8 +123,7 @@ class Keyring(FileBacked, KeyringBackend):
     def set_password(self, service, username, password):
         """Write the password in the file.
         """
-        assoc = (escape_for_ini(service) + '\0' +
-                 escape_for_ini(username)).encode()
+        assoc = self._generate_assoc(service, username)
         # encrypt the password
         password_encrypted = self.encrypt(password.encode('utf-8'), assoc)
         # encode with base64 and add line break to untangle config file
@@ -131,6 +131,12 @@ class Keyring(FileBacked, KeyringBackend):
 
         self._write_config_value(service, username, password_base64)
 
+    def _generate_assoc(self, service, username):
+        """Generate tamper resistant bytestring of associated data
+        """
+        return (escape_for_ini(service) + '\0' +
+                escape_for_ini(username)).encode()
+
     def _write_config_value(self, service, key, value):
         # ensure the file exists
         self._ensure_file_path()
